@@ -36,7 +36,6 @@
 
 #include "config.h"
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <gio/gio.h>
@@ -60,7 +59,6 @@ struct _BluetoothClient {
 
 	GListStore *list_store;
 	Adapter1 *default_adapter;
-	gboolean has_power_state;
 	GDBusObjectManager *manager;
 	GCancellable *cancellable;
 	guint num_adapters;
@@ -77,7 +75,6 @@ enum {
 	PROP_NUM_ADAPTERS,
 	PROP_DEFAULT_ADAPTER,
 	PROP_DEFAULT_ADAPTER_POWERED,
-	PROP_DEFAULT_ADAPTER_STATE,
 	PROP_DEFAULT_ADAPTER_SETUP_MODE,
 	PROP_DEFAULT_ADAPTER_NAME,
 	PROP_DEFAULT_ADAPTER_ADDRESS
@@ -115,26 +112,6 @@ get_device_for_path (BluetoothClient *client,
 	return NULL;
 }
 
-static BluetoothDevice *
-get_device_for_bdaddr (BluetoothClient *client,
-		       const char      *bdaddr)
-{
-	guint n_items, i;
-
-	n_items = g_list_model_get_n_items (G_LIST_MODEL (client->list_store));
-	for (i = 0; i < n_items; i++) {
-		g_autoptr(BluetoothDevice) d = NULL;
-		g_autofree char *s = NULL;
-
-		d = g_list_model_get_item (G_LIST_MODEL (client->list_store), i);
-		g_object_get (G_OBJECT (d), "address", &s, NULL);
-		if (g_ascii_strncasecmp (bdaddr, s, BDADDR_STR_LEN) == 0) {
-			return g_steal_pointer (&d);
-		}
-	}
-	return NULL;
-}
-
 static char **
 device_list_uuids (const gchar * const *uuids)
 {
@@ -187,12 +164,11 @@ phone_oui_to_icon_name (const char *bdaddr)
 }
 
 static const char *
-icon_override (const char    *name,
-	       const char    *bdaddr,
-	       BluetoothType *type)
+icon_override (const char    *bdaddr,
+	       BluetoothType  type)
 {
 	/* audio-card, you're ugly */
-	switch (*type) {
+	switch (type) {
 	case BLUETOOTH_TYPE_HEADSET:
 		return "audio-headset";
 	case BLUETOOTH_TYPE_HEADPHONES:
@@ -205,12 +181,6 @@ icon_override (const char    *name,
 		return "video-display";
 	case BLUETOOTH_TYPE_SCANNER:
 		return "scanner";
-	case BLUETOOTH_TYPE_MOUSE:
-		if (name && strcasestr (name, "tablet")) {
-			*type = BLUETOOTH_TYPE_TABLET;
-			return "input-tablet";
-		}
-		/* fallthrough */
 	case BLUETOOTH_TYPE_REMOTE_CONTROL:
 	case BLUETOOTH_TYPE_WEARABLE:
 	case BLUETOOTH_TYPE_TOY:
@@ -223,14 +193,11 @@ icon_override (const char    *name,
 static void
 device_resolve_type_and_icon (Device1 *device, BluetoothType *type, const char **icon)
 {
-	const char *name;
-
 	g_return_if_fail (type);
 	g_return_if_fail (icon);
 
-	name = device1_get_name (device);
-	if (g_strcmp0 (name, "ION iCade Game Controller") == 0 ||
-	    g_strcmp0 (name, "8Bitdo Zero GamePad") == 0) {
+	if (g_strcmp0 (device1_get_name (device), "ION iCade Game Controller") == 0 ||
+	    g_strcmp0 (device1_get_name (device), "8Bitdo Zero GamePad") == 0) {
 		*type = BLUETOOTH_TYPE_JOYPAD;
 		*icon = "input-gaming";
 		return;
@@ -241,7 +208,7 @@ device_resolve_type_and_icon (Device1 *device, BluetoothType *type, const char *
 	if (*type == 0 || *type == BLUETOOTH_TYPE_ANY)
 		*type = bluetooth_class_to_type (device1_get_class (device));
 
-	*icon = icon_override (name, device1_get_address (device), type);
+	*icon = icon_override (device1_get_address (device), *type);
 
 	if (!*icon) {
 		*icon = device1_get_icon (device);
@@ -402,8 +369,8 @@ unqueue_device_removal (BluetoothClient *client)
 
 			device = g_list_model_get_item (G_LIST_MODEL (client->list_store), i);
 			if (g_str_equal (path, bluetooth_device_get_object_path (device))) {
-				g_list_store_remove (client->list_store, i);
 				g_signal_emit (G_OBJECT (client), signals[DEVICE_REMOVED], 0, path);
+				g_list_store_remove (client->list_store, i);
 				found = TRUE;
 				break;
 			}
@@ -587,10 +554,6 @@ adapter_notify_cb (Adapter1       *adapter,
 		g_object_notify (G_OBJECT (client), "default-adapter-setup-mode");
 	} else if (g_strcmp0 (property, "powered") == 0) {
 		g_object_notify (G_OBJECT (client), "default-adapter-powered");
-		if (!client->has_power_state)
-			g_object_notify (G_OBJECT (client), "default-adapter-state");
-	} else if (g_strcmp0 (property, "power-state") == 0) {
-		g_object_notify (G_OBJECT (client), "default-adapter-state");
 	}
 }
 
@@ -607,7 +570,6 @@ notify_default_adapter_props (BluetoothClient *client)
 	g_object_notify (G_OBJECT (client), "default-adapter");
 	g_object_notify (G_OBJECT (client), "default-adapter-address");
 	g_object_notify (G_OBJECT (client), "default-adapter-powered");
-	g_object_notify (G_OBJECT (client), "default-adapter-state");
 	g_object_notify (G_OBJECT (client), "default-adapter-setup-mode");
 	g_object_notify (G_OBJECT (client), "default-adapter-name");
 }
@@ -742,6 +704,7 @@ adapter_removed (GDBusObjectManager   *manager,
 	if (!was_default)
 		goto out;
 
+
 	new_default_adapter = NULL;
 	object_list = g_dbus_object_manager_get_objects (client->manager);
 	for (l = object_list; l != NULL; l = l->next) {
@@ -1014,7 +977,7 @@ up_device_added_cb (UpClient *up_client,
 		    gpointer  user_data)
 {
 	BluetoothClient *client = user_data;
-	g_autofree char *serial = NULL;
+	g_autofree char *native_path = NULL;
 	g_autoptr(BluetoothDevice) device = NULL;
 	UpDeviceLevel battery_level;
 	double percentage;
@@ -1023,16 +986,16 @@ up_device_added_cb (UpClient *up_client,
 	g_debug ("Considering UPower device %s", up_device_get_object_path (up_device));
 
 	g_object_get (up_device,
-		      "serial", &serial,
+		      "native-path", &native_path,
 		      "battery-level", &battery_level,
 		      "percentage", &percentage,
 		      NULL);
 
-	if (!serial || !bluetooth_verify_address(serial))
+	if (!native_path || !g_str_has_prefix (native_path, "/org/bluez/"))
 		return;
-	device = get_device_for_bdaddr (client, serial);
+	device = get_device_for_path (client, native_path);
 	if (!device) {
-		g_debug ("Could not find bluez device for upower device with serial %s", serial);
+		g_debug ("Could not find bluez device for upower device %s", native_path);
 		return;
 	}
 	g_signal_connect (G_OBJECT (up_device), "notify::battery-level",
@@ -1044,7 +1007,7 @@ up_device_added_cb (UpClient *up_client,
 		battery_type = BLUETOOTH_BATTERY_TYPE_PERCENTAGE;
 	else
 		battery_type = BLUETOOTH_BATTERY_TYPE_COARSE;
-	g_debug ("Applying battery information for %s", serial);
+	g_debug ("Applying battery information for %s", native_path);
 	g_object_set (device,
 		      "battery-type", battery_type,
 		      "battery-level", battery_level,
@@ -1116,7 +1079,6 @@ static void bluetooth_client_init(BluetoothClient *client)
 {
 	client->cancellable = g_cancellable_new ();
 	client->list_store = g_list_store_new (BLUETOOTH_TYPE_DEVICE);
-	client->has_power_state = TRUE;
 
 	g_dbus_object_manager_client_new_for_bus (G_BUS_TYPE_SYSTEM,
 						  G_DBUS_OBJECT_MANAGER_CLIENT_FLAGS_DO_NOT_AUTO_START,
@@ -1215,6 +1177,9 @@ _bluetooth_client_set_default_adapter_discovering (BluetoothClient *client,
 	g_autoptr(GDBusProxy) adapter = NULL;
 	GVariantBuilder builder;
 
+	if (client->discovery_started == discovering)
+		return;
+
 	adapter = _bluetooth_client_get_default_adapter (client);
 	if (adapter == NULL) {
 		g_debug ("%s discovery requested, but no default adapter",
@@ -1223,15 +1188,6 @@ _bluetooth_client_set_default_adapter_discovering (BluetoothClient *client,
 		return;
 	}
 
-	if (!adapter1_get_powered (client->default_adapter) && discovering) {
-		g_debug("Starting discovery requested, but default adapter is unpowered");
-		client->discovery_started = FALSE;
-		return;
-	}
-
-	if (client->discovery_started == discovering)
-		return;
-
 	client->discovery_started = discovering;
 
 	if (discovering) {
@@ -1254,36 +1210,6 @@ _bluetooth_client_set_default_adapter_discovering (BluetoothClient *client,
 	}
 }
 
-static BluetoothAdapterState
-adapter_get_state (BluetoothClient *client)
-{
-	const char *str;
-
-	if (!client->default_adapter)
-		return BLUETOOTH_ADAPTER_STATE_ABSENT;
-
-	str = adapter1_get_power_state (client->default_adapter);
-	if (str) {
-		if (g_str_equal (str, "on"))
-			return BLUETOOTH_ADAPTER_STATE_ON;
-		if (g_str_equal (str, "off") ||
-		    g_str_equal (str, "off-blocked"))
-			return BLUETOOTH_ADAPTER_STATE_OFF;
-		if (g_str_equal (str, "off-enabling"))
-			return BLUETOOTH_ADAPTER_STATE_TURNING_ON;
-		if (g_str_equal (str, "on-disabling"))
-			return BLUETOOTH_ADAPTER_STATE_TURNING_OFF;
-		g_warning_once ("Unexpected adapter PowerState value '%s'", str);
-	} else {
-		client->has_power_state = FALSE;
-	}
-
-	/* Fallback if property is missing, or value is unexpected */
-	return adapter1_get_powered (client->default_adapter) ?
-		BLUETOOTH_ADAPTER_STATE_ON : BLUETOOTH_ADAPTER_STATE_OFF;
-
-}
-
 static void
 bluetooth_client_get_property (GObject        *object,
 			       guint           property_id,
@@ -1304,9 +1230,6 @@ bluetooth_client_get_property (GObject        *object,
 		g_value_set_boolean (value, client->default_adapter ?
 				     adapter1_get_powered (client->default_adapter) : FALSE);
 		break;
-	case PROP_DEFAULT_ADAPTER_STATE:
-		g_value_set_enum (value, adapter_get_state (client));
-		break;
 	case PROP_DEFAULT_ADAPTER_NAME:
 		g_value_set_string (value, client->default_adapter ?
 				    adapter1_get_alias (client->default_adapter) : NULL);
@@ -1440,18 +1363,6 @@ static void bluetooth_client_class_init(BluetoothClientClass *klass)
 					 g_param_spec_boolean ("default-adapter-powered", NULL,
 							      "Whether the default adapter is powered",
 							       FALSE, G_PARAM_READWRITE));
-	/**
-	 * BluetoothClient:default-adapter-state:
-	 *
-	 * The #BluetoothAdapterState of the default Bluetooth adapter. More precise than
-	 * #BluetoothClient:default-adapter-powered.
-	 */
-	g_object_class_install_property (object_class, PROP_DEFAULT_ADAPTER_STATE,
-					 g_param_spec_enum ("default-adapter-state", NULL,
-							    "State of the default adapter",
-							    BLUETOOTH_TYPE_ADAPTER_STATE,
-							    BLUETOOTH_ADAPTER_STATE_ABSENT,
-							    G_PARAM_READABLE));
 	/**
 	 * BluetoothClient:default-adapter-setup-mode:
 	 *
@@ -1785,14 +1696,6 @@ disconnect_callback (GDBusProxy   *proxy,
  * @callback: (scope async): a #GAsyncReadyCallback to call when the connection is complete
  * @user_data: the data to pass to callback function
  *
- * This will start the process of connecting to one of the known-connectable
- * services on the device. This means that it could connect to all the audio
- * services on a headset, but just to the input service on a keyboard.
- *
- * Broadly speaking, this will only have an effect on devices with audio and HID
- * services, and do nothing if the device doesn't have the "connectable"
- * property set.
- *
  * When the connection operation is finished, @callback will be called. You can
  * then call bluetooth_client_connect_service_finish() to get the result of the
  * operation.
